use crate::game::*;

use bevy::prelude::{shape::Quad, *};
use bevy_rapier2d::prelude::*;

pub fn setup(
	commands: &mut Commands,
	meshes: &mut ResMut<Assets<Mesh>>,
	character_meshes: &Res<CharacterMeshes>,
	materials: &mut ResMut<Assets<ColorMaterial>>,
	audio: &Res<crossbeam_channel::Sender<AudioMsg>>,
	asset_server: &Res<AssetServer>,
) {
	commands
		.spawn_bundle(ColorMesh2dBundle {
			mesh: meshes
				.add(Mesh::from(Quad {
					size: Vec2 { x: 800.0, y: 16.0 },
					flip: false,
				}))
				.into(),
			material: materials.add(ColorMaterial::from(Color::GRAY)),
			transform: Transform::from_xyz(0.0, -256.0, 0.0),
			..default()
		})
		.insert(Collider::cuboid(400., 8.))
		.insert(Level);
	commands
		.spawn_bundle(ColorMesh2dBundle {
			mesh: meshes
				.add(Mesh::from(Quad {
					size: Vec2 { x: 400.0, y: 16.0 },
					flip: false,
				}))
				.into(),
			material: materials.add(ColorMaterial::from(Color::GRAY)),
			transform: Transform::from_xyz(256.0, -128.0, 0.0),
			..default()
		})
		.insert(Collider::cuboid(200., 8.))
		.insert(Level);

	spawn_characters(
		commands,
		character_meshes,
		materials,
		audio,
		[
			(Transform::from_xyz(128., 64., 0.), Color::BLUE),
			(Transform::from_xyz(-128., -128., 0.), Color::RED),
			(Transform::from_xyz(0., -128., 0.), Color::GREEN),
		],
	);

	commands
		.spawn_bundle(AbsorbingFilter {
			mesh: ColorMesh2dBundle {
				mesh: meshes
					.add(Mesh::from(Quad {
						size: Vec2 { x: 128.0, y: 16.0 },
						flip: false,
					}))
					.into(),
				material: materials.add(Color::rgba(1., 0., 0., 0.5).into()),
				transform: Transform::from_xyz(0., 0., 2.),
				..Default::default()
			},
			collider: Collider::cuboid(64., 8.),
			sensor: Sensor,
			filter: PassThroughFilter::Absorbing(Color::RED),
		})
		.insert(Level);

	commands
		.spawn_bundle(RotatingFilter {
			sprite: SpriteBundle {
				texture: asset_server.get_handle("bevy.png"),
				transform: Transform::from_xyz(256., -224., 2.),
				..Default::default()
			},
			collider: Collider::ball(32.),
			sensor: Sensor,
			filter: PassThroughFilter::Rotating(45.),
			velocity: Velocity::angular(1.),
			rigid_body: RigidBody::KinematicVelocityBased,
		})
		.insert(Level);
}
